@@ -45,18 +45,18 @@ def parse_helper(lp, args, wf_mode=False):
     :return:
     """
 
-    if sum([bool(x) for x in [args.fw_id, args.name, args.state, args.query]]) != 1:
-        raise ValueError('Please specify exactly one of (fw_id, name, state, query)')
+    if args.fw_id and sum([bool(x) for x in [args.name, args.state, args.query]]) >= 1:
+        raise ValueError('Cannot specify both fw_id and name/state/query)')
 
     query = {}
     if args.fw_id:
         return pw_check([int(x) for x in args.fw_id.split(',')], args)
-    elif args.name:
-        query = {'name': args.name}
-    elif args.state:
-        query = {'state': args.state}
-    elif args.query:
+    if args.query:
         query = ast.literal_eval(args.query)
+    if args.name:
+        query['name'] = args.name
+    if args.state:
+        query['state'] = args.state
 
     if wf_mode:
         return pw_check(lp.get_wf_ids(query), args)
@@ -168,6 +168,14 @@ def lpad():
     refresh_parser.add_argument('-q', '--query', help='query (enclose pymongo-style dict in single-quotes, e.g. \'{"state":"COMPLETED"}\')', default=None)
     refresh_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
 
+    priority_parser = subparsers.add_parser('set_priority', help='modify the priority of one or more FireWorks')
+    priority_parser.add_argument('priority', help='get FW with this fw_id', default=None, type=int)
+    priority_parser.add_argument('-i', '--fw_id', help='fw id or comma separated list of fw ids', default=None)
+    priority_parser.add_argument('-n', '--name', help='name', default=None)
+    priority_parser.add_argument('-s', '--state', help='state ("ARCHIVED", "DEFUSED", "READY", "RESERVED", "FIZZLED", "RUNNING", "COMPLETED")', default=None)
+    priority_parser.add_argument('-q', '--query', help='query (enclose pymongo-style dict in single-quotes, e.g. \'{"state":"COMPLETED"}\')', default=None)
+    priority_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+
     subparsers.add_parser('version', help='Print the version and location of FireWorks installation')
 
     parser.add_argument('-l', '--launchpad_file', help='path to LaunchPad file containing central DB connection info',
@@ -193,6 +201,14 @@ def lpad():
     addscript_parser.add_argument('-w', '--wf_name', help='Workflow name', default=None)
     addscript_parser.add_argument('-d', '--delimiter', help='delimiter for separating scripts', default=',')
 
+    recover_parser = subparsers.add_parser('recover_offline', help='recover offline workflows')
+    recover_parser.add_argument('-i', '--ignore_errors', help='ignore errors', action='store_true')
+
+    forget_parser = subparsers.add_parser('forget_offline', help='forget offline workflows')
+    forget_parser.add_argument('-n', '--name', help='name', default=None)
+    forget_parser.add_argument('-s', '--state', help='state ("ARCHIVED", "DEFUSED", "READY", "RESERVED", "FIZZLED", "RUNNING", "COMPLETED")', default=None)
+    forget_parser.add_argument('-q', '--query', help='query (enclose pymongo-style dict in single-quotes, e.g. \'{"state":"COMPLETED"}\')', default=None)
+
     args = parser.parse_args()
 
     if args.command == 'version':
@@ -227,9 +243,6 @@ def lpad():
                 fwf = Workflow.from_file(filename)
                 lp.add_wf(fwf)
 
-        elif args.command == 'maintain':
-            lp.maintain(args.infinite, args.maintain_interval)
-
         elif args.command == 'tuneup':
             lp.tuneup()
 
@@ -379,6 +392,15 @@ def lpad():
                 lp._refresh_wf(wf, f)
                 print f
 
+        elif args.command == 'set_priority':
+            fw_ids = parse_helper(lp, args, wf_mode=True)
+            for f in fw_ids:
+                lp.set_priority(f, args.priority)
+                lp.m_logger.debug("Set priority of fw_id {}".format(f))
+
+            lp.m_logger.info("Finished setting priorities of {} FireWorks".format(len(fw_ids)))
+
+
         elif args.command == 'webgui':
             os.environ.setdefault("DJANGO_SETTINGS_MODULE", "fireworks.base_site.settings")
             os.environ["FWDB_CONFIG"] = json.dumps(lp.to_dict())
@@ -405,5 +427,23 @@ def lpad():
 
             lp.add_wf(Workflow(fws, links, wf_name))
 
+        elif args.command == 'recover_offline':
+            failed_fws = []
+            for l in lp.offline_runs.find({"completed": False, "deprecated": False}, {"launch_id": 1}):
+                fw = lp.recover_offline(l['launch_id'], args.ignore_errors)
+                if fw:
+                    failed_fws.append(fw)
+
+            lp.m_logger.info("FINISHED recovering offline runs.")
+            if failed_fws:
+                lp.m_logger.info("FAILED to recover offline fw_ids: {}".format(failed_fws))
+
+        elif args.command == 'forget_offline':
+            fw_ids = parse_helper(lp, args)
+            for f in fw_ids:
+                lp.forget_offline(f)
+                print f
+
+
 if __name__ == '__main__':
     lpad()
\ No newline at end of file
